# Figure sizes

ONE_COLUMN_WIDTH = 3.45
TWO_COLUMNS_WIDTH = 7.09
WIDTH = TWO_COLUMNS_WIDTH
HEIGHT = ONE_COLUMN_WIDTH

WIDE_SLIDE_WIDTH = 13
WIDE_SLIDE_HEIGHT = 6.5

# Font sizes

FONTSIZE_SMALL = 5
FONTSIZE = 6
FONTSIZE_BIG = 8

# Marker sizes

MARKERSIZE_SMALL = 5
MARKERSIZE = 10
MARKERSIZE_BIG = 30

# Line sizes

LINEWIDTH = 0.5
LINEWIDTH_THICK = 1



import matplotlib.pyplot as plt
from .figure import AxisManager

from ..vars import ONE_COLUMN_WIDTH, TWO_COLUMNS_WIDTH


def create_figure(
    nrows=1, ncols=1, height=None, width=None, width_ratios=None, height_ratios=None,
):
    if height is None:
        height = ONE_COLUMN_WIDTH
    if width is None:
        if ncols == 1:
            width = ONE_COLUMN_WIDTH
        else:
            width = TWO_COLUMNS_WIDTH
    fig, axs = plt.subplots(
        nrows=nrows,
        ncols=ncols,
        figsize=(width, height),
        gridspec_kw={"width_ratios": width_ratios, "height_ratios": height_ratios},
    )
    fig.patch.set_facecolor("white")
    if nrows == 1 and ncols == 1:
        axs = [[axs]]
    else:
        axs = axs
    return fig, AxisManager(axs)


def save_figure(filename, pad=None):
    if pad is None:
        plt.tight_layout()
    else:
        plt.tight_layout()
    plt.savefig(filename, dpi=600, transparent=False, bbox_inches="tight")



import matplotlib as mpl
import seaborn as sns
import shutil
import os

sns.set_style("ticks")

os.environ["LC_CTYPE"] = "en_US.UTF-8"
import matplotlib.font_manager

FONT = "Arial"

shutil.rmtree(mpl.get_cachedir())

mpl.rcParams["font.family"] = "sans-serif"
mpl.rcParams["font.sans-serif"] = FONT
mpl.rcParams.update({"font.size": 10})
mpl.rcParams["pdf.fonttype"] = 42
mpl.rcParams["ps.fonttype"] = 42
mpl.rcParams.update({"axes.grid": True})

from .sizes.sizes import FontSize, MarkerSize

FontSize()
MarkerSize()

# Relative imports

from .figure import create_figure, save_figure
from .figure.figure import label
from .colors.colors import NamedColors, ContinuousColorMap, NamedColorMaps
from .vars import *



import os
from setuptools.command.build_py import build_py as build_py_orig
from setuptools.command.sdist import sdist as sdist_orig

STATIC_VERSION_FILE = "_clean_static_version.py"

# No public API
__all__ = []

package_root = os.path.dirname(os.path.realpath(__file__))
package_name = os.path.basename(package_root)
distr_root = os.path.dirname(package_root)
# If the package is inside a "src" directory the
# distribution root is 1 level up.
if os.path.split(distr_root)[1] == "src":
    _package_root_inside_src = True
    distr_root = os.path.dirname(distr_root)
else:
    _package_root_inside_src = False

script_path = os.path.dirname(os.path.abspath(__file__))
clean_version_file = os.path.join(script_path, STATIC_VERSION_FILE)
with open(clean_version_file, "r") as f:
    version = f.read().split('"')[1]

__version__ = version


def _write_version(fname):
    # This could be a hard link, so try to delete it first.  Is there any way
    # to do this atomically together with opening?
    try:
        os.remove(fname)
    except OSError:
        pass
    with open(fname, "w") as f:
        f.write(
            "# This file has been created by setup.py.\n"
            "version = '{}'\n".format(__version__)
        )


class _build_py(build_py_orig):
    def run(self):
        super().run()
        _write_version(os.path.join(self.build_lib, package_name, STATIC_VERSION_FILE))


class _sdist(sdist_orig):
    def make_release_tree(self, base_dir, files):
        super().make_release_tree(base_dir, files)
        if _package_root_inside_src:
            p = os.path.join("src", package_name)
        else:
            p = package_name
        _write_version(os.path.join(base_dir, p, STATIC_VERSION_FILE))


cmdclass = dict(sdist=_sdist, build_py=_build_py)



import random
import numpy as np

import matplotlib as mpl
from matplotlib import cm
from cmcrameri import cm as cms
import palettable
from matplotlib import colors

from sklearn.preprocessing import QuantileTransformer

from ..utils import lighten_color, set_transparency


CONTINUOUS_CMAP_PERCENITLE_CUTS = (1, 99)


class NamedColors(object):
    def __init__(self):
        self._colors = palettable.cartocolors.qualitative.Prism_9.mpl_colors
        self.red = self._colors[7]
        self.blue = self._colors[1]
        self.green = self._colors[3]
        self.orange = self._colors[6]
        self.purple = self._colors[8]
        self.yellow = self._colors[5]
        self.gray = tuple(list(colors.to_rgba("lightgray"))[:3])
        self.white = tuple(list(colors.to_rgba("white"))[:3])
        self.black = tuple(list(colors.to_rgba("black"))[:3])

    def _get(self, color_name, alpha=None, lighten=None):
        if color_name == "red":
            return self.red
        if color_name == "blue":
            return self.blue
        if color_name == "green":
            return self.green
        if color_name == "orange":
            return self.orange
        if color_name == "purple":
            return self.purple
        if color_name == "yellow":
            return self.yellow
        if color_name == "gray":
            return self.gray
        if color_name == "white":
            return self.white
        if color_name == "black":
            return self.black

    def get(self, color_name, alpha=None, lighten=None):
        color = self._get(color_name)
        if lighten is not None:
            color = lighten_color(color, factor=lighten)
        if alpha is not None:
            color = set_transparency(color, alpha)
        return color


class NamedColorMaps(object):
    def __init__(self, scientific=True):
        self._scientific = scientific
        if scientific:
            self.viridis = cms.imola
            self.spectral = cms.roma
            self.coolwarm = cms.vik
        else:
            self.viridis = cm.get_cmap("viridis")
            self.spectral = cm.get_cmap("Spectral")
            self.coolwarm = cm.get_cmap("coolwarm")

    def get(self, cmap_name):
        if cmap_name == "viridis":
            return self.viridis
        if cmap_name == "spectral":
            return self.spectral
        if cmap_name == "coolwarm":
            return self.coolwarm


class Palette(object):
    def __init__(self, shuffle=False):
        self.colors = palettable.cartocolors.qualitative.Prism_10.mpl_colors
        self.is_shuffled = shuffle
        if shuffle:
            idxs = [i for i in range(len(self.colors))]
            random.shuffle(idxs)
            self.colors = [self.colors[i] for i in idxs]
        self.cur_i = 0

    def next(self):
        if self.cur_i > len(self.colors):
            self.cur_i = 0
        color = self.colors[self.cur_i]
        self.cur_i += 1
        return color

    def sample(self, n):
        colors = []
        for _ in range(n):
            colors += [self.next()]
        return colors


class ContinuousColorMap(object):
    def __init__(self, cmap="spectral", transformation="uniform", ascending=True):
        if type(cmap) is str:
            self.cmap = NamedColorMaps().get(cmap)
        else:
            self.cmap = cmap
        self.transformation = transformation
        self.ascending = ascending

    def fit(self, data):
        data = np.array(data)
        if not self.ascending:
            data = -data
        if self.transformation is not None:
            values = np.array(data).reshape(-1, 1)
            self.transformer = QuantileTransformer(
                n_quantiles=min(len(values), 1000),
                output_distribution=self.transformation,
            )
            self.transformer.fit(values)
            if self.transformation == "uniform":
                self.vmin = 0
                self.vmax = 1
            else:
                values = self.transformer.transform(values).ravel()
                self.vmin = np.percentile(values, CONTINUOUS_CMAP_PERCENITLE_CUTS[0])
                self.vmax = np.percentile(values, CONTINUOUS_CMAP_PERCENITLE_CUTS[1])
        else:
            self.transformer = None
            values = np.array(data)
            self.vmin = np.percentile(values, CONTINUOUS_CMAP_PERCENITLE_CUTS[0])
            self.vmax = np.percentile(values, CONTINUOUS_CMAP_PERCENITLE_CUTS[1])
        self.color_normalizer = mpl.colors.Normalize(vmin=self.vmin, vmax=self.vmax)

    def transform(self, data):
        data = np.array(data)
        if not self.ascending:
            data = -data
        if self.transformer:
            values = np.array(data).reshape(-1, 1)
            values = self.transformer.transform(values).ravel()
        else:
            values = np.array(data)
        colors = [self.cmap(self.color_normalizer(v)) for v in values]
        return colors

    def get(self, data, alpha=None, lighten=None):
        colors = self.transform(data)
        if lighten is not None:
            colors = [lighten_color(c) for c in colors]
        if alpha is not None:
            colors = [set_transparency(c) for c in colors]
        return colors

    def sample(self, n, shuffle=False):
        values = np.linspace(0, 1, n)
        norm = mpl.colors.Normalize(vmin=0, vmax=1)
        values = [norm(x) for x in values]
        if shuffle:
            random.shuffle(values)
        return [self.cmap(x) for x in values]


class CategoricalColorMap(object):
    pass



from ..colors.colors import Palette
from ..vars import LINEWIDTH, FONTSIZE_BIG


def stylize(ax):
    ax.set_prop_cycle("color", Palette().colors)
    ax.set_xlabel("X-axis / Units")
    ax.set_ylabel("Y-axis / Units")
    ax.set_title("Plot title")
    try:
        ax.grid(b=True, linewidth=LINEWIDTH)
    except:
        ax.grid(visible=True, linewidth=LINEWIDTH)
    ax.xaxis.set_tick_params(width=LINEWIDTH)
    ax.yaxis.set_tick_params(width=LINEWIDTH)
    return ax


def label(ax, xlabel=None, ylabel=None, title=None, abc=None):
    if xlabel is not None:
        ax.set_xlabel(xlabel)
    if xlabel is not None:
        ax.set_ylabel(ylabel)
    if title is not None:
        ax.set_title(title)
    if abc is not None:
        ax.set_title(abc, loc="left", fontweight="bold", fontsize=FONTSIZE_BIG)


class AxisManager(object):
    def __init__(self, axs):
        if type(axs) is list:
            self.axs_flat = axs[0]
        else:
            self.axs_flat = axs.flatten()
        self.axs = axs
        self.current_i = 0

    def __getitem__(self, key):
        if type(key) is int:
            i = key
            ax = self.axs_flat[i]
            self.current_i = i + 1
        else:
            i, j = key
            ax = self.axs[i, j]
            self.current_i = j * (i + 1) + 1
        ax = stylize(ax)
        return ax

    def restart(self):
        self.current_i = 0

    def next(self):
        ax = self.axs_flat[self.current_i]
        self.current_i += 1
        ax = stylize(ax)
        return ax



import matplotlib.pyplot as plt
import matplotlib as mpl
import numpy as np

from ..vars import FONTSIZE_SMALL, FONTSIZE, LINEWIDTH, MARKERSIZE


class FontSize(object):
    def __init__(self):
        self._set_rc_params()

    def _set_rc_params(self):
        plt.rc("font", size=FONTSIZE_SMALL)
        plt.rc("axes", titlesize=FONTSIZE)
        plt.rc("axes", labelsize=FONTSIZE)
        plt.rc("xtick", labelsize=FONTSIZE)
        plt.rc("ytick", labelsize=FONTSIZE)
        plt.rc("legend", fontsize=FONTSIZE)
        plt.rc("figure", titlesize=FONTSIZE)


class MarkerSize(object):
    def __init__(self):
        self._set_rc_params()

    def _set_rc_params(self):
        mpl.rcParams["axes.linewidth"] = LINEWIDTH
        mpl.rcParams["lines.markersize"] = np.sqrt(MARKERSIZE)
        mpl.rcParams["lines.linewidth"] = LINEWIDTH



import matplotlib.colors as mc
import colorsys


def lighten_color(rgb_color, factor=1):
    c = list(rgb_color)
    if len(c) == 4:
        alpha = c[-1]
        c = c[:3]
    else:
        alpha = None
    c = tuple(c)
    c = colorsys.rgb_to_hls(*mc.to_rgb(c))
    c = colorsys.hls_to_rgb(c[0], 1 - factor * (1 - c[1]), c[2])
    c = list(c)
    if alpha is not None:
        c += [alpha]
    return tuple(c)


def set_transparency(rgb_color, alpha):
    rgb_color = list(rgb_color)
    if len(rgb_color) == 4:
        rgb_color[-1] = alpha
    else:
        rgb_color += [alpha]
    return tuple(rgb_color)
